{{
File: Test Propeller Eeprom.spin
Version: 0.675

Developed for forthcoming Propeller Education Kit Lab: EEPROM Datalogging and I2C

See Test Propeller Eeprom Docs.spin for explanation and instructions.  To view this
object, press, and the file should appear in the Object Info window's Object Explorer
pane.  Double-click it to open.

}}

CON                                              ' Constant declarations

  ' System clock settings - 80 MHz
  _clkmode = xtal1 + pll16x                      ' Low speed external crystal 16x PLL
  _xinfreq = 5_000_000                           ' Crystal input frequency is 5 MHz
                     
VAR                                              ' Variable declarations

  ''Global variables including value, a 32 long array that gets backed up and restored.
  long value[32], start, end, eeaddr, count, size

OBJ                                              ' Object declarations

  appdocs : "Test Propeller Eeprom Docs"         ' Docuemntation for this application
  eeprom  : "Propeller Eeprom"                   ' Propeller EEPROM object
  menu    : "Menu"                               ' HyperTerminal menu for this app

PUB Main | select                                '' Main method
                                                 ''
  '' Displays user prompts and calls methods based on user input.

  menu.Init                                      ' Inialize Menu object

  repeat                                         ' Main repeat loop
    case menu.Main & %11011111                   ' Mask selection to upper case & execute
      "C" : longfill(@value, 0, 32)              ' "C": Clear all array elements
      "D" : Display                              ' "D": Display array elements
      "M" : Modify                               ' "M": Modify array elements
      "B" : Backup                               ' "B": Back up array to EEPROM
      "L" : Load                                 ' "R": Restore array from EEPROM
      "T" : ArrayToEe                            ' "T": Array To EEPROM starting at... 
      "F" : ArrayFromEe                          ' "F": Array From EEPROM starting at...
      "W" : Write                                ' "W": Similar to PBASIC WRITE command
      "R" : Read                                 ' "R": Similar to PBASIC READ command
      other : menu.TryAgain                      ' Wrong menu key? Error + "try again"
    menu.KeyPress                                ' Message: Press any key to continue

PUB Display | i                                  '' Display selected array elements
                                                 ''
  '' Prompts user to choose start and end values.  Displays array elements from start
  '' element to end element'

  menu.GetLimits(@start, @end)                   ' Get start & end elements from user
  repeat i from start to end                     ' Repeat loop from start to end
    menu.DisplayElement(value[i], i)             ' Display array element & index
    
PUB Modify | i                                   '' Modify array elements
                                                 ''
  '' Prompts user to choose start and end of the range of array elements to be modified.
  '' Prompts user to enter each element in the selected range and updates each element.  

  menu.GetLimits(@start, @end)                   ' Get start & end elements from user
  repeat i from start to end                     ' Loop through each element
    value[i] := menu.GetElement(i)               ' Get user value & store in element
   
PUB Backup                                       '' Back up entire array to EEPROM
                                                 ''
  '' This method's eeprom.VarBackup call backs up a range of contiguous variables from
  '' global RAM to their corresponding addresses in non volotile EEPROM.  Note that 3 gets
  '' added to the VarBackup method's endAddr parameter because value[31] points to the
  '' address of the first of the 4 bytes that make up the value[31] long.  
                                                 
  eeprom.VarBackup(@value, @value[31] + 3)       ' Copy value array from RAM to EEPROM
   
PUB Load                                         '' Load entire array from EEPROM
                                                 ''
  '' This method's eeprom.VarRestore call backs up a range of contiguous variables from
  '' global RAM to their corresponding addresses in non volotile EEPROM.  
                                                 
  eeprom.VarRestore(@value, @value[31] + 3)      ' Copy array from EEPROM to RAM
   
PUB ArrayToEe                                    '' Array range to EEPROM start address
                                                 ''
  '' Prompts user to enter start and end array elements, and then copies the values stored
  '' by those elements to another location in EEPROM, starting at eeAddr.  Note that
  '' eeprom.FromRam gets called instead of eeprom.VarBackup
                                                 
  menu.GetEeInfo(@start, @end, @eeAddr)                    ' Get parameters from user
  eeprom.FromRam(@value[start], @value[end] + 3, eeAddr)   ' Use paramters in FromRam call
   
PUB ArrayFromEe                                  '' EEPROM range at start address to array
                                                 ''
  '' Prompts user to enter start and end array elements, and then copies the values stored
  '' starting at eeAddr value[start]..value[end].  
                                                 
  menu.GetEeInfo(@start, @end, @eeAddr)                    ' Get paramters from user
  eeprom.ToRam(@value[start], @value[end] + 3, eeAddr)     ' Use parameters in ToRam call
   
PUB Write | i, temp                              '' Write individual values to EEPROM
                                                 ''
  '' Prompts user for EEPROM starting address, number of values to be entered and variable
  '' size.  Then, prompts user for values and stores each one to EEPROM locations building
  '' from larger to smaller addresses.  
                                                 
  menu.EeParams(@eeAddr, @count, @size)
  repeat i from eeAddr - size + 1 to eeAddr - (count * size) + 1 step size
    temp := menu.getElement(i)
    eeprom.FromRam(@temp.byte[0], @temp.byte[size-1], i)
   
PUB Read | i, temp

  temp~
  menu.EeParams(@eeAddr, @count, @size)
  repeat i from eeAddr - size + 1 to eeAddr - (count * size) + 1 step size
    eeprom.ToRam(@temp.byte[0], @temp.byte[size - 1], i)
    menu.DisplayElement(temp, i)
   